@@ -23,8 +23,9 @@ import os
 from os.path import dirname, exists, expanduser
 import re
 import shlex
-from subprocess import call, Popen, PIPE, DEVNULL
+import subprocess
 import sys
+from threading import Thread
 import webbrowser
 
 import urwid
@@ -81,7 +82,7 @@ def splittext(text, search, attr):
 
     """
     if search:
-        pat = re.compile("({})".format(re.escape(search)), re.IGNORECASE)
+        pat = re.compile(f"({re.escape(search)})", re.IGNORECASE)
     else:
         return text
     final = pat.split(text)
@@ -106,9 +107,11 @@ class URLChooser:
                      '7': self._digits,
                      '8': self._digits,
                      '9': self._digits,
+                     'a': self._add_url,
                      'C': self._clipboard,
                      'c': self._context,
                      'ctrl l': self._clear_screen,
+                     'd': self._del_url,
                      'f1': self._help_menu,
                      'G': self._bottom,
                      'g': self._top,
@@ -116,6 +119,8 @@ class URLChooser:
                      'k': self._up,
                      'P': self._clipboard_pri,
                      'l': self._link_handler,
+                     'o': self._open_queue,
+                     'O': self._open_queue_win,
                      'p': self._palette,
                      'Q': self._quit,
                      'q': self._quit,
@@ -146,20 +151,20 @@ class URLChooser:
                ('urlref:url', 'white', 'black', 'standout'),
                ('url:sel', 'black', 'light gray', 'bold')]
         # Boruch's colorized palette
-        colorized =[('header','brown','black','standout'),
-                    ('footer','white','dark red','standout'),
-                    ('search','white','dark green','standout'),
-                    ('msgtext','light cyan','black'),
-                    ('msgtext:ellipses','light gray','black'),
-                    ('urlref:number:braces','light gray','black'),
-                    ('urlref:number','yellow','black','standout'),
-                    ('urlref:url','dark green','black','standout'),
-                    ('url:sel','white','black','')]
+        colorized = [('header', 'brown', 'black', 'standout'),
+                     ('footer', 'white', 'dark red', 'standout'),
+                     ('search', 'white', 'dark green', 'standout'),
+                     ('msgtext', 'light cyan', 'black'),
+                     ('msgtext:ellipses', 'light gray', 'black'),
+                     ('urlref:number:braces', 'light gray', 'black'),
+                     ('urlref:number', 'yellow', 'black', 'standout'),
+                     ('urlref:url', 'dark green', 'black', 'standout'),
+                     ('url:sel', 'white', 'black', '')]
         self.palettes.update([("default", default), ("bw", blw), ("colorized", colorized)])
         if genconf is True:
             self._config_create()
         try:
-            with open(self.conf, 'r') as conf_file:
+            with open(self.conf, 'r', encoding=sys.getdefaultencoding()) as conf_file:
                 data = json.load(conf_file)
                 try:
                     for pal_name, pal in data['palettes'].items():
@@ -172,7 +177,7 @@ class URLChooser:
                         if value:
                             if value == "open_url":
                                 urwid.Button._command_map._command[key] = 'activate'
-                            value = getattr(self, "_{}".format(value))
+                            value = getattr(self, f"_{value}")
                         else:
                             del self.keys[key]
                             continue
@@ -182,12 +187,13 @@ class URLChooser:
         except FileNotFoundError:
             pass
         try:
-            call(['xdg-open'], stdout=DEVNULL)
+            subprocess.run(['xdg-open'], check=False, stdout=subprocess.DEVNULL)
             self.xdg = True
         except OSError:
             self.xdg = False
         self.shorten = shorten
         self.compact = compact
+        self.queue = []
         self.run = run
         self.runsafe = runsafe
         self.single = single
@@ -219,7 +225,8 @@ class URLChooser:
         self.header = (":: F1 - help/keybindings :: "
                        "q - quit :: "
                        "/ - search :: "
-                       "URL opening mode - {}")
+                       "URL opening mode - {} :: "
+                       "Queue - {}")
         self.link_open_modes = ["Web Browser", "Xdg-Open"] if self.xdg is True else ["Web Browser"]
         if self.runsafe:
             self.link_open_modes.insert(0, self.runsafe)
@@ -228,7 +235,7 @@ class URLChooser:
         self.nohelp = nohelp
         if nohelp is False:
             self.headerwid = urwid.AttrMap(urwid.Text(
-                self.header.format(self.link_open_modes[0])), 'header')
+                self.header.format(self.link_open_modes[0], len(self.queue))), 'header')
         else:
             self.headerwid = None
         self.top = urwid.Frame(listbox, self.headerwid)
@@ -270,7 +277,7 @@ class URLChooser:
         """
         for j, k in enumerate(keys):
             if self.search is True:
-                text = "Search: {}".format(self.search_string)
+                text = f"Search: {self.search_string}"
                 if k == 'enter':
                     # Catch 'enter' key to prevent opening URL in mkbrowseto
                     self.enter = True
@@ -339,19 +346,84 @@ class URLChooser:
     def _open_url(self):
         """<Enter> or <space>"""
         load_text = "Loading URL..." if self.link_open_modes[0] != (self.run or self.runsafe) \
-            else "Executing: {}".format(self.run or self.runsafe)
+            else f"Executing: {self.run or self.runsafe}"
         if os.environ.get('BROWSER') not in ['elinks', 'links', 'w3m', 'lynx']:
             self._footer_display(load_text, 5)
 
+    def _background_queue(self, mode):
+        """Open URLs in background"""
+        for url in self.queue:
+            self.mkbrowseto(url, thread=True, mode=mode)()
+        self.draw_screen()
+
+    def _queue(self, mode=2):
+        """Open all URLs in queue
+
+            Args: mode - 2 for new tab, 1 for new window
+
+        """
+        load_text = "Loading URLs in queue..." \
+            if self.link_open_modes[0] != (self.run or self.runsafe) \
+            else f"Executing: {self.run or self.runsafe}"
+        if os.environ.get('BROWSER') in ['elinks', 'links', 'w3m', 'lynx']:
+            self._footer_display("Opening multiple links not support in text browsers", 5)
+        else:
+            self._footer_display(load_text, 5)
+        thr = Thread(target=self._background_queue, args=(mode,))
+        thr.start()
+        self.queue = []
+        self.headerwid = urwid.AttrMap(urwid.Text(
+            self.header.format(self.link_open_modes[0], len(self.queue))), 'header')
+        self.top.base_widget.header = self.headerwid
+
+    def _open_queue(self):
+        """o (new tab)"""
+        if self.queue:
+            self._queue()
+
+    def _open_queue_win(self):
+        """O (new window)"""
+        if self.queue:
+            self._queue(1)
+
+    def _add_url(self):
+        """a"""
+        fpo = self.top.base_widget.body.focus_position
+        url_idx = len([i for i in self.items[:fpo + 1]
+                       if isinstance(i, urwid.Columns)]) - 1
+        if self.compact is False and fpo <= 1:
+            return
+        self.queue.append(self.urls[url_idx])
+        self.queue = list(set(self.queue))
+        self.headerwid = urwid.AttrMap(urwid.Text(
+            self.header.format(self.link_open_modes[0], len(self.queue))), 'header')
+        self.top.base_widget.header = self.headerwid
+
+    def _del_url(self):
+        """d"""
+        fpo = self.top.base_widget.body.focus_position
+        url_idx = len([i for i in self.items[:fpo + 1]
+                       if isinstance(i, urwid.Columns)]) - 1
+        if self.compact is False and fpo <= 1:
+            return
+        try:
+            self.queue.remove(self.urls[url_idx])
+            self.headerwid = urwid.AttrMap(urwid.Text(
+                self.header.format(self.link_open_modes[0], len(self.queue))), 'header')
+            self.top.base_widget.header = self.headerwid
+        except ValueError:
+            pass
+
     def _help_menu(self):
         """F1"""
         if self.help_menu is False:
             self.focus_pos_saved = self.top.base_widget.body.focus_position
-            help_men = "\n".join(["{} - {}".format(i, j.__name__.strip('_'))
+            help_men = "\n".join([f"{i} - {j.__name__.strip('_')}"
                                   for i, j in self.keys.items() if j.__name__ !=
                                   '_digits'])
             help_men = "KEYBINDINGS\n" + help_men + "\n<0-9> - Jump to item"
             docs = ("OPTIONS\n"
+                    "add_url       -- add URL to queue\n"
                     "all_escape    -- toggle unescape all URLs\n"
                     "all_shorten   -- toggle shorten all URLs\n"
                     "bottom        -- move cursor to last item\n"
@@ -362,10 +434,13 @@ class URLChooser:
                     "                 selection using xsel/xclip\n"
                     "config_create -- create ~/.config/urlscan/config.json\n"
                     "context       -- show/hide context\n"
+                    "del_url       -- delete URL from queue\n"
                     "down          -- cursor down\n"
                     "help_menu     -- show/hide help menu\n"
                     "link_handler  -- cycle through xdg-open, webbrowser \n"
                     "                 and user-defined function\n"
+                    "open_queue    -- open all URLs in queue\n"
+                    "open_queue_win-- open all URLs in queue in new window\n"
                     "open_url      -- open selected URL\n"
                     "palette       -- cycle through palettes\n"
                     "quit          -- quit\n"
@@ -413,9 +488,11 @@ class URLChooser:
                     self.items.index(self.items[max(int(self.number) - 1, 0)])
         except IndexError:
             self.number = self.number[:-1]
+        except ValueError:
+            pass
         self.top.base_widget.keypress(self.size, "")  # Trick urwid into redisplaying the cursor
         if self.number:
-            self._footer_display("Selection: {}".format(self.number), 1)
+            self._footer_display(f"Selection: {self.number}", 1)
 
     def _clear_screen(self):
         """ Ctrl-l """
@@ -524,10 +601,13 @@ class URLChooser:
         cmds = COPY_COMMANDS_PRIMARY if pri else COPY_COMMANDS
         for cmd in cmds:
             try:
-                proc = Popen(shlex.split(cmd), stdin=PIPE, stdout=DEVNULL, stderr=DEVNULL)
-                proc.communicate(input=url.encode(sys.getdefaultencoding()))
-                self._footer_display("Copied url to {} selection".format(
-                    "primary" if pri is True else "clipboard"), 5)
+                subprocess.run(shlex.split(cmd),
+                               check=False,
+                               input=url.encode(sys.getdefaultencoding()),
+                               stdout=subprocess.DEVNULL,
+                               stderr=subprocess.DEVNULL)
+                self._footer_display("Copied url to "
+                                     f"{'primary' if pri is True else 'clipboard'} selection", 5)
             except OSError:
                 continue
             if self.single is True:
@@ -557,7 +637,7 @@ class URLChooser:
             os.makedirs(dirname(expanduser(self.conf)), exist_ok=True)
             keys = dict(zip(self.keys.keys(),
                             [i.__name__.strip('_') for i in self.keys.values()]))
-            with open(expanduser(self.conf), 'w') as pals:
+            with open(expanduser(self.conf), 'w', encoding=sys.getdefaultencoding()) as pals:
                 pals.writelines(json.dumps({"palettes": self.palettes, "keys": keys},
                                            indent=4))
             print("Created ~/.config/urlscan/config.json")
@@ -579,7 +659,7 @@ class URLChooser:
 
         """
         self.number = ""  # Clear URL selection number
-        text = "Search: {}".format(self.search_string)
+        text = f"Search: {self.search_string}"
         if self.search_string:
             footer = 'search'
         else:
@@ -604,7 +684,7 @@ class URLChooser:
         """ Search - search URLs and text.
 
         """
-        text = "Search: {}".format(self.search_string)
+        text = f"Search: {self.search_string}"
         footerwid = urwid.AttrMap(urwid.Text(text), 'footer')
         self.top.base_widget.footer = footerwid
         search_items = []
@@ -661,15 +741,19 @@ class URLChooser:
         self.link_open_modes.insert(0, mode)
         if self.nohelp is False:
             self.headerwid = urwid.AttrMap(urwid.Text(
-                self.header.format(self.link_open_modes[0])), 'header')
+                self.header.format(self.link_open_modes[0], len(self.queue))), 'header')
             self.top.base_widget.header = self.headerwid
 
-    def mkbrowseto(self, url):
+    def mkbrowseto(self, url, thread=False, mode=0):
         """Create the urwid callback function to open the web browser or call
         another function with the URL.
 
         """
-        def browse(*args):
+        def browse(*args):  # pylint: disable=unused-argument
+            # These 3 lines prevent any stderr messages from webbrowser or xdg
+            savout = os.dup(2)
+            os.close(2)
+            os.open(os.devnull, os.O_RDWR)
             # double ()() to ensure self.search evaluated at runtime, not when
             # browse() is _created_. [0] is self.search, [1] is self.enter
             # self.enter prevents opening URL when in search mode
@@ -678,26 +762,30 @@ class URLChooser:
                     self.search = False
                     self.enter = False
             elif self.link_open_modes[0] == "Web Browser":
-                webbrowser.open(url)
+                webbrowser.open(url, new=mode)
             elif self.link_open_modes[0] == "Xdg-Open":
-                run = 'xdg-open "{}"'.format(url)
-                process = Popen(shlex.split(run), stdout=PIPE, stdin=PIPE)
+                subprocess.run(shlex.split(f'xdg-open "{url}"'), check=False)
             elif self.link_open_modes[0] == self.runsafe:
                 if self.pipe:
-                    process = Popen(shlex.split(self.runsafe), stdout=PIPE, stdin=PIPE)
-                    process.communicate(input=url.encode(sys.getdefaultencoding()))
+                    subprocess.run(shlex.split(self.runsafe),
+                                   check=False,
+                                   input=url.encode(sys.getdefaultencoding()))
                 else:
                     cmd = [i.format(url) for i in shlex.split(self.runsafe)]
-                    Popen(cmd).communicate()
+                    subprocess.run(cmd, check=False)
             elif self.link_open_modes[0] == self.run and self.pipe:
-                process = Popen(shlex.split(self.run), stdout=PIPE, stdin=PIPE)
-                process.communicate(input=url.encode(sys.getdefaultencoding()))
+                subprocess.run(shlex.split(self.run),
+                               check=False,
+                               input=url.encode(sys.getdefaultencoding()))
             else:
-                Popen(self.run.format(url), shell=True).communicate()
+                subprocess.run(self.run.format(url), check=False, shell=True)
 
             if self.single is True:
                 self._quit()
-            self.draw_screen()
+            # Restore normal stderr
+            os.dup2(savout, 2)
+            if thread is False:
+                self.draw_screen()
         return browse
 
     def process_urls(self, extractedurls, dedupe, shorten):
